@@ -40,14 +40,12 @@ program so
   call Poisson_Jacobi(nx,ny,dx,dy,rho,1.0e-6,tp,psi,bound)
 !  call Poisson_GauSei(nx,ny,dx,dy,rho,1.0e-6,tp,psi)
 
+  call color_setting( 10, cmin, cmax )
+
   call DclOpenGraphics(IWS)
 
-  call color_setting( 75, cmin, cmax )
-
-  call DclNewFrame
-
   call Dcl_2D_cont_shade( 'solution', x, y, psi, psi, contmin, contmax,  &
-  &    cmin, cmax, 'x-axis', 'y-axis', form_typec, form_types )
+  &    cmin, cmax, 'x-axis', 'y-axis', form_typec, form_types, color_num=10 )
 
   call DclCloseGraphics
 
