@@ -2,11 +2,9 @@ module analy  ! ׻̥ǡѲϥ⥸塼
 
 contains
 
-subroutine div( nx, ny, dx, dy, u, v, val, undeff )
+subroutine div( x, y, u, v, val, undeff, hx, hy )
 ! 2ȯ׻롼
 ! 
-! dx, dy  u, v ϶ִθ֤, Ǥդ2ȯ׻ǽ
-!
 ! $\frac{\partial u}{\partial x} +\frac{\partial v}{\partial y} $ 
 ! 2 ʬǽ񤭴,  $(i,j)$ Ǥȯ
 ! $\frac{u_{i+1,j}-u_{i-1,j}}{2dx} + \frac{v_{i,j+1}-v_{i,j-1}}{2dy} $
@@ -15,17 +13,28 @@ subroutine div( nx, ny, dx, dy, u, v, val, undeff )
 ! .
 ! ŪֳֳʻҤΥǡΤߤ׻ǤʤΤ, ֳ֤Υǡ
 ! ֳ֤ޤʤԤäƤ餳Υ롼ƤӽФɬפ.
+! , ǤդγʻҺɸϤб뤿, ˥󥰥եץ
+! ɲä, ߤϻѤƤʤ.
   implicit none
-  integer, intent(in) :: nx  ! ǿ 1 
-  integer, intent(in) :: ny  ! ǿ 2 
-  real, intent(in) :: dx  ! 1 ܤʬʻҴֳ [m]
-  real, intent(in) :: dy  ! 2 ܤʬʻҴֳ [m]
-  real, intent(in) :: u(nx,ny)  ! x б 2 ٥ȥʬ
-  real, intent(in) :: v(nx,ny)  ! y б 2 ٥ȥʬ
-  real, intent(inout) :: val(nx,ny)  ! 2ȯ
+  real, intent(in) :: x(:)  ! x ζֺɸ [m]
+  real, intent(in) :: y(:)  ! y ζֺɸ [m]
+  real, intent(in) :: u(size(x),size(y))  ! x б 2 ٥ȥʬ
+  real, intent(in) :: v(size(x),size(y))  ! y б 2 ٥ȥʬ
+  real, intent(inout) :: val(size(x),size(y))  ! 2ȯ
   real, intent(in), optional :: undeff
+  real, intent(in), optional :: hx(size(x),size(y))  ! x Υ
+  real, intent(in), optional :: hy(size(x),size(y))  ! y Υ
   integer :: i   ! 졼ź
   integer :: j   ! 졼ź
+  integer :: nx  ! ǿ 1 
+  integer :: ny  ! ǿ 2 
+  real :: dx  ! 1 ܤʬʻҴֳ [m]
+  real :: dy  ! 2 ܤʬʻҴֳ [m]
+
+  dx=x(2)-x(1)
+  dy=y(2)-y(1)
+  nx=size(x)
+  ny=size(y)
 
   if(present(undeff))then
      do j=2,ny-1
@@ -126,12 +135,13 @@ subroutine div( nx, ny, dx, dy, u, v, val, undeff )
 end subroutine div
 
 
-subroutine curl( nx, ny, dx, dy, u, v, val, undeff )
+subroutine curl( x, y, u, v, val, undeff, hx, hy, ord )
 ! 2 ٷ׻롼
 !
-! dx, dy  u, v ϶ִθ֤, Ǥդ2٤׻ǽ
-! nx, ny μμ㤤ɬ¤٤뤳.
-! dx, dy  nx, ny ν֤ˤĤƤбطϤʤ.
+! x, y μμ㤤ɬ¤٤뤳.
+!
+! u, v ϶ִθ֤, Ǥդ2٤׻ǽ
+! , du/dz-dw/dx ׻ȤΤ, (x,z,u,w) ν֤, ord ץ false.
 !
 ! $\frac{\partial v}{\partial x} -\frac{\partial u}{\partial y} $ 
 ! 2 ʬǽ񤭴,  $(i,j)$ Ǥȯ
@@ -141,17 +151,38 @@ subroutine curl( nx, ny, dx, dy, u, v, val, undeff )
 ! .
 ! ŪֳֳʻҤΥǡΤߤ׻ǤʤΤ, ֳ֤Υǡ
 ! ֳ֤ޤʤԤäƤ餳Υ롼ƤӽФɬפ.
+! , ǤդγʻҺɸϤб뤿, ˥󥰥եץ
+! ɲä, ߤϻѤƤʤ.
   implicit none
-  integer, intent(in) :: nx  ! ǿ 1 
-  integer, intent(in) :: ny  ! ǿ 2 
-  real, intent(in) :: dx  ! 1 ܤʬʻҴֳ [m]
-  real, intent(in) :: dy  ! 2 ܤʬʻҴֳ [m]
-  real, intent(in) :: u(nx,ny)  ! x б 2 ٥ȥʬ
-  real, intent(in) :: v(nx,ny)  ! y б 2 ٥ȥʬ
-  real, intent(inout) :: val(nx,ny)  ! 2
+  real, intent(in) :: x(:)  ! x ζֺɸ [m]
+  real, intent(in) :: y(:)  ! y ζֺɸ [m]
+  real, intent(in) :: u(size(x),size(y))  ! x б 2 ٥ȥʬ
+  real, intent(in) :: v(size(x),size(y))  ! y б 2 ٥ȥʬ
+  real, intent(inout) :: val(size(x),size(y))  ! 2
   real, intent(in), optional :: undeff
+  real, intent(in), optional :: hx(size(x),size(y))  ! x Υ
+  real, intent(in), optional :: hy(size(x),size(y))  ! y Υ
+  logical, intent(in),  optional :: ord  ! ʬν֤ؤ륪ץ.
+                 ! true ʤ, ؤʤ, false ʤ, ؤ.
+                 ! ǥեȤ true, du/dz-dw/dx ׻ȤΤѤ.
   integer :: i   ! 졼ź
   integer :: j   ! 졼ź
+  integer :: nx  ! ǿ 1 
+  integer :: ny  ! ǿ 2 
+  real :: dx  ! 1 ܤʬʻҴֳ [m]
+  real :: dy  ! 2 ܤʬʻҴֳ [m]
+  logical :: order
+
+  if(present(ord))then
+     order=ord
+  else
+     order=.true.
+  end if
+
+  dx=x(2)-x(1)
+  dy=y(2)-y(1)
+  nx=size(x)
+  ny=size(y)
 
   if(present(undeff))then
      do j=2,ny-1
@@ -248,12 +279,20 @@ subroutine curl( nx, ny, dx, dy, u, v, val, undeff )
      val(nx,ny)=(v(nx,ny)-v(nx-1,ny))/dx-(u(nx,ny)-u(nx,ny-1))/dy
   end if
 
+  if(order.eqv..false.)then
+     do j=1,ny
+        do i=1,nx
+           val(i,j)=-val(i,j)
+        end do
+     end do
+  end if
+
 end subroutine curl
 
 !-----------------------------------------
 !-----------------------------------------
 
-subroutine div_3d( nx, ny, nz, dx, dy, dz, u, v, w, val, undeff )
+subroutine div_3d( x, y, z, u, v, w, val, undeff, hx, hy, hz )
 ! 3ȯ׻롼
 !
 ! $\frac{\partial u}{\partial x} +\frac{\partial v}{\partial y} +\frac{\partial w}{\partial z} $ 
@@ -264,21 +303,36 @@ subroutine div_3d( nx, ny, nz, dx, dy, dz, u, v, w, val, undeff )
 ! .
 ! ŪֳֳʻҤΥǡΤߤ׻ǤʤΤ, ֳ֤Υǡ
 ! ֳ֤ޤʤԤäƤ餳Υ롼ƤӽФɬפ.
+! , ǤդγʻҺɸϤб뤿, ˥󥰥եץ
+! ɲä, ߤϻѤƤʤ.
   implicit none
-  integer, intent(in) :: nx  ! ǿ 1 
-  integer, intent(in) :: ny  ! ǿ 2 
-  integer, intent(in) :: nz  ! ǿ 3 
-  real, intent(in) :: dx  ! 1 ܤʬʻҴֳ [m]
-  real, intent(in) :: dy  ! 2 ܤʬʻҴֳ [m]
-  real, intent(in) :: dz  ! 3 ܤʬʻҴֳ [m]
-  real, intent(in) :: u(nx,ny,nz)  ! x б 3 ٥ȥʬ
-  real, intent(in) :: v(nx,ny,nz)  ! y б 3 ٥ȥʬ
-  real, intent(in) :: w(nx,ny,nz)  ! z б 3 ٥ȥʬ
-  real, intent(inout) :: val(nx,ny,nz)  ! 3 ȯ
+  real, intent(in) :: x(:)  ! x ζֺɸ [m]
+  real, intent(in) :: y(:)  ! y ζֺɸ [m]
+  real, intent(in) :: z(:)  ! z ζֺɸ [m]
+  real, intent(in) :: u(size(x),size(y),size(z))  ! x б 2 ٥ȥʬ
+  real, intent(in) :: v(size(x),size(y),size(z))  ! y б 2 ٥ȥʬ
+  real, intent(in) :: w(size(x),size(y),size(z))  ! y б 2 ٥ȥʬ
+  real, intent(inout) :: val(size(x),size(y),size(z))  ! 3 ȯ
   real, intent(in), optional :: undeff
+  real, intent(in), optional :: hx(size(x),size(y),size(z))  ! x Υ
+  real, intent(in), optional :: hy(size(x),size(y),size(z))  ! y Υ
+  real, intent(in), optional :: hz(size(x),size(y),size(z))  ! z Υ
   integer :: i   ! 졼ź
   integer :: j   ! 졼ź
   integer :: k   ! 졼ź
+  integer :: nx  ! ǿ 1 
+  integer :: ny  ! ǿ 2 
+  integer :: nz  ! ǿ 3 
+  real :: dx  ! 1 ܤʬʻҴֳ [m]
+  real :: dy  ! 2 ܤʬʻҴֳ [m]
+  real :: dz  ! 3 ܤʬʻҴֳ [m]
+
+  dx=x(2)-x(1)
+  dy=y(2)-y(1)
+  dz=z(2)-z(1)
+  nx=size(x)
+  ny=size(y)
+  nz=size(z)
 
   if(present(undeff))then
 !$omp parallel default(shared)
@@ -321,8 +375,8 @@ subroutine div_3d( nx, ny, nz, dx, dy, dz, u, v, w, val, undeff )
 !   ȯ­碌뤳ȤǷ׻.
 !-- z ˿ľ ---
   if(present(undeff))then
-     call div( nx, ny, dx, dy, u(:,:,1), v(:,:,1), val(:,:,1), undeff )
-     call div( nx, ny, dx, dy, u(:,:,nz), v(:,:,nz), val(:,:,nz), undeff )
+     call div( x, y, u(:,:,1), v(:,:,1), val(:,:,1), undeff )
+     call div( x, y, u(:,:,nz), v(:,:,nz), val(:,:,nz), undeff )
      do j=2,ny-1
         do i=2,nx-1
            if(val(i,j,1)==undeff.or.w(i,j,2)==undeff.or.w(i,j,1)==undeff)then
@@ -339,8 +393,8 @@ subroutine div_3d( nx, ny, nz, dx, dy, dz, u, v, w, val, undeff )
         end do
      end do
   else
-     call div( nx, ny, dx, dy, u(:,:,1), v(:,:,1), val(:,:,1) )
-     call div( nx, ny, dx, dy, u(:,:,nz), v(:,:,nz), val(:,:,nz) )
+     call div( x, y, u(:,:,1), v(:,:,1), val(:,:,1) )
+     call div( x, y, u(:,:,nz), v(:,:,nz), val(:,:,nz) )
      do j=2,ny-1
         do i=2,nx-1
            val(i,j,1)=val(i,j,1)+(w(i,j,2)-w(i,j,1))/dz
@@ -351,8 +405,8 @@ subroutine div_3d( nx, ny, nz, dx, dy, dz, u, v, w, val, undeff )
 
 !-- y ˿ľ ---
   if(present(undeff))then
-     call div( nx, nz, dz, dx, w(:,1,:), u(:,1,:), val(:,1,:), undeff )
-     call div( nx, nz, dz, dx, w(:,ny,:), u(:,ny,:), val(:,ny,:), undeff )
+     call div( x, z, u(:,1,:), w(:,1,:), val(:,1,:), undeff )
+     call div( x, z, u(:,ny,:), w(:,ny,:), val(:,ny,:), undeff )
      do k=2,nz-1
         do i=2,nx-1
            if(val(i,1,k)==undeff.or.v(i,2,k)==undeff.or.v(i,1,k)==undeff)then
@@ -369,8 +423,8 @@ subroutine div_3d( nx, ny, nz, dx, dy, dz, u, v, w, val, undeff )
         end do
      end do
   else
-     call div( nx, nz, dz, dx, w(:,1,:), u(:,1,:), val(:,1,:) )
-     call div( nx, nz, dz, dx, w(:,ny,:), u(:,ny,:), val(:,ny,:) )
+     call div( x, z, u(:,1,:), w(:,1,:), val(:,1,:) )
+     call div( x, z, u(:,ny,:), w(:,ny,:), val(:,ny,:) )
      do k=2,nz-1
         do i=2,nx-1
            val(i,1,k)=val(i,1,k)+(v(i,2,k)-v(i,1,k))/dy
@@ -381,8 +435,8 @@ subroutine div_3d( nx, ny, nz, dx, dy, dz, u, v, w, val, undeff )
 
 !-- x ˿ľ ---
   if(present(undeff))then
-     call div( ny, nz, dy, dz, v(1,:,:), w(1,:,:), val(1,:,:), undeff )
-     call div( ny, nz, dy, dz, v(nx,:,:), w(nx,:,:), val(nx,:,:), undeff )
+     call div( y, z, v(1,:,:), w(1,:,:), val(1,:,:), undeff )
+     call div( y, z, v(nx,:,:), w(nx,:,:), val(nx,:,:), undeff )
      do k=2,nz-1
         do j=2,ny-1
            if(val(1,j,k)==undeff.or.u(2,j,k)==undeff.or.u(1,j,k)==undeff)then
@@ -399,8 +453,8 @@ subroutine div_3d( nx, ny, nz, dx, dy, dz, u, v, w, val, undeff )
         end do
      end do
   else
-     call div( ny, nz, dy, dz, v(1,:,:), w(1,:,:), val(1,:,:) )
-     call div( ny, nz, dy, dz, v(nx,:,:), w(nx,:,:), val(nx,:,:) )
+     call div( y, z, v(1,:,:), w(1,:,:), val(1,:,:) )
+     call div( y, z, v(nx,:,:), w(nx,:,:), val(nx,:,:) )
      do k=2,nz-1
         do j=2,ny-1
            val(1,j,k)=val(1,j,k)+(u(2,j,k)-u(1,j,k))/dx
@@ -701,7 +755,7 @@ end subroutine
 !-----------------------------------------
 !-----------------------------------------
 
-subroutine rotate( nx, ny, nz, dx, dy, dz, u, v, w, zeta, eta, xi, undeff )
+subroutine rotate( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
 ! 3 ٤׻.
 ! ν֤ϱϤ x, y, z ΥǥȺɸ,
 ! б٥ȥʬ u, v, w ,
@@ -711,28 +765,45 @@ subroutine rotate( nx, ny, nz, dx, dy, dz, u, v, w, zeta, eta, xi, undeff )
 ! ٥٥ȥ׻Τ, ̤ȶʬΥƷ׻
 ! ɬפϤʤ.
 ! ̤˿ľʱ٥٥ȥ curl ǶޤǹθƷ׻ǽǤ.
+! , ǤդγʻҺɸϤб뤿, ˥󥰥եץ
+! ɲä, ߤϻѤƤʤ.
   implicit none
-  integer, intent(in) :: nx  ! ǿ 1 
-  integer, intent(in) :: ny  ! ǿ 2 
-  integer, intent(in) :: nz  ! ǿ 3 
-  real, intent(in) :: dx  ! 1 ܤʬʻҴֳ [m]
-  real, intent(in) :: dy  ! 2 ܤʬʻҴֳ [m]
-  real, intent(in) :: dz  ! 3 ܤʬʻҴֳ [m]
-  real, intent(in) :: u(nx,ny,nz)  ! ٥ȥ x ʬ
-  real, intent(in) :: v(nx,ny,nz)  ! ٥ȥ y ʬ
-  real, intent(in) :: w(nx,ny,nz)  ! ٥ȥ z ʬ
-  real, intent(inout) :: zeta(nx,ny,nz)  ! ٥٥ȥ x ʬ
-  real, intent(inout) :: eta(nx,ny,nz)  ! ٥٥ȥ y ʬ
-  real, intent(inout) :: xi(nx,ny,nz)  ! ٥٥ȥ z ʬ
+  real, intent(in) :: x(:)  ! x ζֺɸ [m]
+  real, intent(in) :: y(:)  ! y ζֺɸ [m]
+  real, intent(in) :: z(:)  ! z ζֺɸ [m]
+  real, intent(in) :: u(size(x),size(y),size(z))  ! x б 2 ٥ȥʬ
+  real, intent(in) :: v(size(x),size(y),size(z))  ! y б 2 ٥ȥʬ
+  real, intent(in) :: w(size(x),size(y),size(z))  ! y б 2 ٥ȥʬ
+  real, intent(inout) :: zeta(size(x),size(y),size(z))  ! ٥٥ȥ x ʬ
+  real, intent(inout) :: eta(size(x),size(y),size(z))  ! ٥٥ȥ y ʬ
+  real, intent(inout) :: xi(size(x),size(y),size(z))  ! ٥٥ȥ z ʬ
   real, intent(in), optional :: undeff
-  integer :: i, j, k
+  real, intent(in), optional :: hx(size(x),size(y),size(z))  ! x Υ
+  real, intent(in), optional :: hy(size(x),size(y),size(z))  ! y Υ
+  real, intent(in), optional :: hz(size(x),size(y),size(z))  ! z Υ
+  integer :: i   ! 졼ź
+  integer :: j   ! 졼ź
+  integer :: k   ! 졼ź
+  integer :: nx  ! ǿ 1 
+  integer :: ny  ! ǿ 2 
+  integer :: nz  ! ǿ 3 
+  real :: dx  ! 1 ܤʬʻҴֳ [m]
+  real :: dy  ! 2 ܤʬʻҴֳ [m]
+  real :: dz  ! 3 ܤʬʻҴֳ [m]
+
+  dx=x(2)-x(1)
+  dy=y(2)-y(1)
+  dz=z(2)-z(1)
+  nx=size(x)
+  ny=size(y)
+  nz=size(z)
 
   if(present(undeff))then
 !$omp parallel default(shared)
 !$omp do private(i)
 ! x ˿ľ̾Ǥ x α٥٥ȥ 3 Ƿ׻.
      do i=1,nx
-        call curl( ny, nz, dy, dz, v(i,:,:), w(i,:,:), zeta(i,:,:), undeff )
+        call curl( y, z, v(i,:,:), w(i,:,:), zeta(i,:,:), undeff )
      end do
 
 !$omp end do
@@ -742,7 +813,7 @@ subroutine rotate( nx, ny, nz, dx, dy, dz, u, v, w, zeta, eta, xi, undeff )
 !$omp do private(i)
 ! x ˿ľ̾Ǥ x α٥٥ȥ 3 Ƿ׻.
      do i=1,nx
-        call curl( ny, nz, dy, dz, v(i,:,:), w(i,:,:), zeta(i,:,:) )
+        call curl( y, z, v(i,:,:), w(i,:,:), zeta(i,:,:) )
      end do
 !$omp end do
 !$omp end parallel
@@ -755,7 +826,7 @@ subroutine rotate( nx, ny, nz, dx, dy, dz, u, v, w, zeta, eta, xi, undeff )
 !$omp parallel default(shared)
 !$omp do private(j)
      do j=1,ny
-        call curl( nx, nz, dz, dx, w(:,j,:), u(:,j,:), eta(:,j,:), undeff )
+        call curl( x, z, u(:,j,:), w(:,j,:), eta(:,j,:), undeff, ord=.false. )
      end do
 !$omp end do
 !$omp end parallel
@@ -764,7 +835,7 @@ subroutine rotate( nx, ny, nz, dx, dy, dz, u, v, w, zeta, eta, xi, undeff )
 !$omp parallel default(shared)
 !$omp do private(j)
      do j=1,ny
-        call curl( nx, nz, dz, dx, w(:,j,:), u(:,j,:), eta(:,j,:) )
+        call curl( x, z, u(:,j,:), w(:,j,:), eta(:,j,:), ord=.false. )
      end do
 !$omp end do
 !$omp end parallel
@@ -777,7 +848,7 @@ subroutine rotate( nx, ny, nz, dx, dy, dz, u, v, w, zeta, eta, xi, undeff )
 !$omp parallel default(shared)
 !$omp do private(k)
      do k=1,nz
-        call curl( nx, ny, dx, dy, u(:,:,k), v(:,:,k), xi(:,:,k), undeff )
+        call curl( x, y, u(:,:,k), v(:,:,k), xi(:,:,k), undeff )
      end do
 
 !$omp end do
@@ -786,7 +857,7 @@ subroutine rotate( nx, ny, nz, dx, dy, dz, u, v, w, zeta, eta, xi, undeff )
 !$omp parallel default(shared)
 !$omp do private(k)
      do k=1,nz
-        call curl( nx, ny, dx, dy, u(:,:,k), v(:,:,k), xi(:,:,k) )
+        call curl( x, y, u(:,:,k), v(:,:,k), xi(:,:,k) )
      end do
 
 !$omp end do
@@ -798,7 +869,7 @@ end subroutine
 !-----------------------------------------
 !-----------------------------------------
 
-subroutine grad_1d( nx, dx, u, val, undef )
+subroutine grad_1d( x, u, val, undef, hx )
 ! 1 Υ顼ѿθۤ׻
 ! $\frac{\partial p}{\partial x} $ 
 ! 2 ʬǽ񤭴,  $(i)$ Ǥθۤ
@@ -809,12 +880,17 @@ subroutine grad_1d( nx, dx, u, val, undef )
 ! ŪֳֳʻҤΥǡΤߤ׻ǤʤΤ, ֳ֤Υǡ
 ! ֳ֤ޤʤԤäƤ餳Υ롼ƤӽФɬפ.
   implicit none
-  integer, intent(in) :: nx  ! ǿ
-  real, intent(in) :: dx  ! ʻҴֳ [m]
-  real, intent(in) :: u(nx)  ! ζб 1 顼
-  real, intent(inout) :: val(nx)  ! 顼ͤ x θ
+  real, intent(in) :: x(:)  ! ֺɸ
+  real, intent(in) :: u(size(x))  ! ζб 1 顼
+  real, intent(inout) :: val(size(x))  ! 顼ͤ x θ
   real, intent(in), optional :: undef
+  real, intent(in), optional :: hx(size(x))  ! x Υ
   integer :: i  ! 졼ź
+  integer :: nx  ! ǿ
+  real :: dx  ! ʻҴֳ [m]
+
+  nx=size(x)
+  dx=x(2)-x(1)
 
   if(present(undef))then
      do i=2,nx-1
@@ -849,7 +925,7 @@ end subroutine grad_1d
 
 
 !-- 1 顼ۤΥ롼Ѥ 2 ۤΥ٥ȥ׻ ---
-subroutine grad_vec_2d( nx, ny, dx, dy, u, valx, valy, undeff )
+subroutine grad_vec_2d( x, y, u, valx, valy, undeff, hx, hy )
   !-------------------------------------------------------------------
   ! $\nabla _hp =\left(\frac{\partial p}{\partial x} ,\; \frac{\partial p}{\partial y} \right) $ 
   ! 2 ʬǽ񤭴,  $(i,j)$ Ǥθۤ
@@ -862,32 +938,41 @@ subroutine grad_vec_2d( nx, ny, dx, dy, u, valx, valy, undeff )
   ! Ѥ 1 ۷׻롼 2 ƤӽФȤˤƤ.
   !-------------------------------------------------------------------
   implicit none
-  integer, intent(in) :: nx  ! x ǿ(size ؿǼưŪ˷׻)
-  integer, intent(in) :: ny  ! y ǿ(size ؿǼưŪ˷׻)
-  real, intent(in) :: dx  ! x γʻֳ
-  real, intent(in) :: dy  ! y γʻֳ
-  real, intent(in) :: u(nx,ny)  ! ۤȤ 2 顼ʬ
-  real, intent(inout) :: valx(nx,ny)  ! ׻줿 y  2 ۥ٥ȥ
-  real, intent(inout) :: valy(nx,ny)  ! ׻줿 y  2 ۥ٥ȥ
+  real, intent(in) :: x(:)  ! x κɸѿ [m]
+  real, intent(in) :: y(:)  ! y κɸѿ [m]
+  real, intent(in) :: u(size(x),size(y))  ! ۤȤ 2 顼ʬ
+  real, intent(inout) :: valx(size(x),size(y))  ! ׻줿 y  2 ۥ٥ȥ
+  real, intent(inout) :: valy(size(x),size(y))  ! ׻줿 y  2 ۥ٥ȥ
   real, intent(in), optional :: undeff
+  real, intent(in), optional :: hx(size(x),size(y))  ! x Υ
+  real, intent(in), optional :: hy(size(x),size(y))  ! y Υ
   integer :: i   ! 졼ź
   integer :: j   ! 졼ź
+  integer :: nx  ! x ǿ(size ؿǼưŪ˷׻)
+  integer :: ny  ! y ǿ(size ؿǼưŪ˷׻)
+  real :: dx  ! x γʻֳ
+  real :: dy  ! y γʻֳ
+
+  nx=size(x)
+  ny=size(y)
+  dx=x(2)-x(1)
+  dy=y(2)-y(1)
 
   if(present(undeff))then
      do i=1,ny
-        call grad_1d(nx, dx, u(:,i), valx(:,i), undeff)
+        call grad_1d(x, u(:,i), valx(:,i), undeff)
      end do
 
      do i=1,nx
-        call grad_1d(ny, dy, u(i,:), valy(i,:), undeff)
+        call grad_1d(y, u(i,:), valy(i,:), undeff)
      end do
   else
      do i=1,ny
-        call grad_1d(nx, dx, u(:,i), valx(:,i) )
+        call grad_1d(x, u(:,i), valx(:,i) )
      end do
 
      do i=1,nx
-        call grad_1d(ny, dy, u(i,:), valy(i,:) )
+        call grad_1d(y, u(i,:), valy(i,:) )
      end do
   end if
 
@@ -896,7 +981,7 @@ end subroutine grad_vec_2d
 
 
 !-- 1 顼ۤΥ롼Ѥ 3 ۤΥ٥ȥ׻ ---
-subroutine grad_vec_3d( nx, ny, nz, dx, dy, dz, u, valx, valy, valz, undeff )
+subroutine grad_vec_3d( x, y, z, u, valx, valy, valz, undeff, hx, hy, hz )
   !-------------------------------------------------------------------
   ! $\nabla p =\left(\frac{\partial p}{\partial x} ,\; \frac{\partial p}{\partial y} ,\; \frac{\partial p}{\partial z} \right) $ 
   ! 2 ʬǽ񤭴,  $(i,j,k)$ Ǥθۤ
@@ -909,60 +994,138 @@ subroutine grad_vec_3d( nx, ny, nz, dx, dy, dz, u, valx, valy, valz, undeff )
   ! Ѥ 1 ۷׻롼 3 ƤӽФȤˤƤ.
   !-------------------------------------------------------------------
   implicit none
-  integer, intent(in) :: nx  ! x ǿ(size ؿǼưŪ˷׻)
-  integer, intent(in) :: ny  ! y ǿ(size ؿǼưŪ˷׻)
-  integer, intent(in) :: nz  ! z ǿ(size ؿǼưŪ˷׻)
-  real, intent(in) :: dx  ! x γʻֳ
-  real, intent(in) :: dy  ! y γʻֳ
-  real, intent(in) :: dz  ! z γʻֳ
-  real, intent(in) :: u(nx,ny,nz)  ! ۤȤ 3 顼ʬ
-  real, intent(inout) :: valx(nx,ny,nz)  ! ׻줿 x  3 ۥ٥ȥ
-  real, intent(inout) :: valy(nx,ny,nz)  ! ׻줿 y  3 ۥ٥ȥ
-  real, intent(inout) :: valz(nx,ny,nz)  ! ׻줿 z  3 ۥ٥ȥ
+  real, intent(in) :: x(:)  ! x κɸѿ [m]
+  real, intent(in) :: y(:)  ! y κɸѿ [m]
+  real, intent(in) :: z(:)  ! z κɸѿ [m]
+  real, intent(in) :: u(size(x),size(y),size(z))  ! ۤȤ 2 顼ʬ
+  real, intent(inout) :: valx(size(x),size(y),size(z))  ! ׻줿 y  2 ۥ٥ȥ
+  real, intent(inout) :: valy(size(x),size(y),size(z))  ! ׻줿 y  2 ۥ٥ȥ
+  real, intent(inout) :: valz(size(x),size(y),size(z))  ! ׻줿 z  2 ۥ٥ȥ
   real, intent(in), optional :: undeff
+  real, intent(in), optional :: hx(size(x),size(y),size(z))  ! x Υ
+  real, intent(in), optional :: hy(size(x),size(y),size(z))  ! y Υ
+  real, intent(in), optional :: hz(size(x),size(y),size(z))  ! z Υ
   integer :: i   ! 졼ź
   integer :: j   ! 졼ź
+  integer :: k   ! 졼ź
+  integer :: nx  ! x ǿ(size ؿǼưŪ˷׻)
+  integer :: ny  ! y ǿ(size ؿǼưŪ˷׻)
+  integer :: nz  ! z ǿ(size ؿǼưŪ˷׻)
+  real :: dx  ! x γʻֳ
+  real :: dy  ! y γʻֳ
+  real :: dz  ! z γʻֳ
+
+  nx=size(x)
+  ny=size(y)
+  nz=size(z)
+  dx=x(2)-x(1)
+  dy=y(2)-y(1)
+  dz=z(2)-z(1)
 
   if(present(undeff))then
      do i=1,nz
         do j=1,ny
-           call grad_1d(nx, dx, u(:,j,i), valx(:,j,i),undeff)
+           call grad_1d(x, u(:,j,i), valx(:,j,i),undeff)
         end do
      end do
 
      do i=1,nz
         do j=1,nx
-           call grad_1d(ny, dy, u(j,:,i), valy(j,:,i),undeff)
+           call grad_1d(y, u(j,:,i), valy(j,:,i),undeff)
         end do
      end do
 
      do i=1,ny
         do j=1,nx
-           call grad_1d(nz, dz, u(j,i,:), valz(j,i,:),undeff)
+           call grad_1d(z, u(j,i,:), valz(j,i,:),undeff)
         end do
      end do
   else
      do i=1,nz
         do j=1,ny
-           call grad_1d(nx, dx, u(:,j,i), valx(:,j,i))
+           call grad_1d(x, u(:,j,i), valx(:,j,i))
         end do
      end do
 
      do i=1,nz
         do j=1,nx
-           call grad_1d(ny, dy, u(j,:,i), valy(j,:,i))
+           call grad_1d(y, u(j,:,i), valy(j,:,i))
         end do
      end do
 
      do i=1,ny
         do j=1,nx
-           call grad_1d(nz, dz, u(j,i,:), valz(j,i,:))
+           call grad_1d(z, u(j,i,:), valz(j,i,:))
         end do
      end do
   end if
 
 end subroutine grad_vec_3d
 
+!-----------------------------------------
+!-----------------------------------------
+
+subroutine laplacian_1d( x, u, val, undef, hx )
+! 1 Υ顼ѿΥץ饷׻
+! $\frac{\partial ^2p}{\partial x^2} $ 
+! 2 ʬǽ񤭴,  $(i)$ Ǥθۤ
+! $\frac{p_{i+1}+p_{i-1}-2p_i}{dx^2} $
+! ȤǤ. Ѥ 1 ץ饷׻.
+! ǡ­ʤΰǤ, undef .
+! option  undef Ƥʤ, 0.0 .
+! ŪֳֳʻҤΥǡΤߤ׻ǤʤΤ, ֳ֤Υǡ
+! ֳ֤ޤʤԤäƤ餳Υ롼ƤӽФɬפ.
+  implicit none
+  real, intent(in) :: x(:)  ! x κɸѿ [m]
+  real, intent(in) :: u(size(x))  ! ζб 1 顼
+  real, intent(inout) :: val(size(x))  ! 顼ͤ x θ
+  real, intent(in), optional :: undef
+  real, intent(in), optional :: hx(size(x))  ! x Υ
+  integer :: i  ! 졼ź
+  integer :: nx  ! ǿ
+  real :: dx  ! ʻҴֳ [m]
+
+  nx=size(x)
+  dx=x(2)-x(1)
+
+  if(present(undef))then
+     do i=2,nx-1
+        if(u(i+1)==undef.or.u(i-1)==undef.or.u(i)==undef)then
+           val(i)=undef
+        else
+           val(i)=(u(i+1)+u(i-1)-2.0*u(i))/(dx*dx)
+        end if
+     end do
+!-- ǡΤʤξüν ---
+!     if(u(1)==undef.or.u(2)==undef)then
+!        val(1)=undef
+!     else
+!        val(1)=(u(2)-u(1))/dx
+!     end if
+!     if(u(nx)==undef.or.u(nx-1)==undef)then
+!        val(nx)=undef
+!     else
+!        val(nx)=(u(nx)-u(nx-1))/dx
+!     end if
+     val(1)=undef
+     val(nx)=undef
+  else
+     do i=2,nx-1
+        val(i)=(u(i+1)+u(i-1)-2.0*u(i))/(dx*dx)
+     end do
+!-- ǡΤʤξüν ---
+!     val(1)=(u(2)-u(1))/dx
+!     val(nx)=(u(nx)-u(nx-1))/dx
+     val(1)=0.0
+     val(nx)=0.0
+  end if
+
+end subroutine laplacian_1d
+
+!-----------------------------------------
+!-----------------------------------------
+
+
 subroutine abst(x,y,z,dis)  ! 3 ٥ȥͤ׻롼
   ! Ĵ뤳Ȥˤ, 2 Ǥη׻ǽ.
   implicit none
