@@ -623,18 +623,18 @@ real function qrsg_2_dbz( rho, qr, qs, qg )
   coef=kaijo(6.0)*(((epi-1.0)/(epi+2.0))/((epw-1.0)/(epw+2.0))**2)
 
   if(present(qs))then
-     z_tmp=z_tmp+coef*((rhos/rhow)**2)*ns0*((rho*qs/(pi*rhos*ns0))**1.75)*1.0e18
+     z_tmp=z_tmp+coef*((rhos/rhow)**2)*ns0*((rho*qs/(pi*rhos*ns0))**1.75)!*1.0e18
   end if
 
   if(present(qg))then
-     z_tmp=z_tmp+coef*((rhog/rhow)**2)*ng0*((rho*qg/(pi*rhog*ng0))**1.75)*1.0e18
+     z_tmp=z_tmp+coef*((rhog/rhow)**2)*ng0*((rho*qg/(pi*rhog*ng0))**1.75)!*1.0e18
   end if
 
-  z_tmp=z_tmp+nr0*((rho*qr/(pi*rhow*nr0))**1.75)*1.0e18
+  z_tmp=z_tmp+nr0*((rho*qr/(pi*rhow*nr0))**1.75)!*1.0e18
   z_tmp=coef*z_tmp
 
   if(z_tmp>0.0)then
-     qrsg_2_dbz=10.0*log10(z_tmp)
+     qrsg_2_dbz=180.0+10.0*log10(z_tmp)
   else
      qrsg_2_dbz=0.0
   end if
